/**
 * OW2 Util
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: ISpecificClassMetadata.java 4389 2008-12-15 13:48:57Z alitokmen $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.scan.api.metadata.specific;

import java.util.Collection;
import java.util.List;

import org.ow2.util.scan.api.metadata.structures.JClass;
import org.ow2.util.scan.api.metadata.structures.JField;
import org.ow2.util.scan.api.metadata.structures.JMethod;

/**
 * Interface of a specific class metadata.
 * @param <SC> interface of a specific class metadata
 * @param <SM> interface of a specific method metadata
 * @param <SF> interface of a specific field metadata
 * @author Gael Lalire
 */
public interface ISpecificClassMetadata<
    SC extends ISpecificClassMetadata<SC, SM, SF>,
    SM extends ISpecificMethodMetadata<SC, SM, SF>,
    SF extends ISpecificFieldMetadata<SC, SM, SF>> {

    /**
     * add a method metadata.
     * @param methodAnnotationMetadata the method metadata
     */
    void addSpecificMethodMetadata(final SM methodAnnotationMetadata);

    /**
     * @param jMethod key of the map of methods annotations.
     * @return method annotation metadata of a given method.
     */
    SM getSpecificMethodMetadata(final JMethod jMethod);

    /**
     * @param methodName the name of the method to search
     * @return list of method annotation metadata for the given method name
     */
    List<? extends SM> searchSpecificMethodMetadata(final String methodName);

    /**
     * @param fieldName the name of the field to search
     * @return list of field annotation metadata for the given field name
     */
    List<? extends SF> searchSpecificFieldMetadata(final String fieldName);

    /**
     * Get collections of methods annotation metadata.
     * @return collections of methods annotation metadata.
     */
    Collection<? extends SM> getSpecificMethodMetadataCollection();

    /**
     * Add field annotation metadata for a given Bean.
     * @param fieldAnnotationMetadata metadata of a field.
     */
    void addSpecificFieldMetadata(final SF fieldAnnotationMetadata);

    /**
     * @param jField key of the map of fields annotations.
     * @return field annotation metadata of a given method.
     */
    SF getSpecificFieldMetadata(final JField jField);

    /**
     * Get collections of fields annotation metadata.
     * @return collections of fields annotation metadata.
     */
    Collection<? extends SF> getSpecificFieldMetadataCollection();

    /**
     * @return class representation
     */
    JClass getJClass();

}
